# -*- coding: utf-8 -*-
"""
    Colubrid Execute
    ================

    Since the 1.0 version of colubrid this is only a small helper module
    which uses paster to serve the application.
"""
from __future__ import generators
import sys
import os
import time
from paste import httpserver
from threading import Thread

__all__ = ('execute', 'run_test', 'stop_test')


class StaticExports(object):

    def __init__(self, app, exports):
        self.application = app
        self.exports = exports

    def serve_file(self, filename, start_response):
        from mimetypes import guess_type
        guessed_type = guess_type(filename)
        if guessed_type[0] is None:
            mime_type = 'text/plain'
        else:
            if guessed_type[1] is None:
                mime_type = guessed_type[0]
            else:
                mime_type = ';charset='.join(guessed_type)
            start_response('200 OK', [('Content-Type', mime_type)])
            fp = file(filename)
            yield fp.read()
            fp.close()

    def __call__(self, environ, start_response):
        path_info = environ.get('PATH_INFO', '')
        for search_path, file_path in self.exports.iteritems():
            if not search_path.endswith('/'):
                search_path += '/'
            if path_info.startswith(search_path):
                real_path = os.path.join(file_path, path_info[len(search_path):])
                if os.path.exists(real_path) and os.path.isfile(real_path):
                    return self.serve_file(real_path, start_response)
        return self.application(environ, start_response)


class SilentWsgiHandler(httpserver.WSGIHandler):
    """An silent version of httpserver.WSGIHandler"""
    
    def log_request(self, code='-', size='-'):
        pass


class ServerTestThread(Thread):

    def __init__(self, run_server):
        super(ServerTestThread, self).__init__()
        self.running = False
        self.run_server = run_server

    def stop(self):
        self.running = False

    def run(self):
        try:
            srv = self.run_server()
        except Exception, e:
            self.running = False
            print e
        else:
            self.running = True
            while self.running:
                srv.handle_request()
            self.running = False

server_test_thread = None


def run_test(app, hostname='localhost', port=8080):
    global server_test_thread
    try:
        server_test_thread
    except NameError:
        server_test_thread = None
    if not server_test_thread is None:
        # stop currently running test
        stop_test()
    run = lambda: httpserver.serve(app, host=hostname, port=str(port),
                                   handler=SilentWsgiHandler, start_loop=False)
    server_test_thread = ServerTestThread(run)
    server_test_thread.start()


def stop_test():
    global server_test_thread
    running = False
    try:
        running = not server_test_thread is None
    except:
        pass
    if running:
        raise RuntimeError('Test Server not running')
    server_test_thread.stop()
    while server_test_thread.running:
        time.sleep(1)
    server_test_thread = None


def execute(app=None, debug=True, hostname='localhost', port=8080,
            reload=False):
    if app is None:
        frm = sys._getframe().f_back
        if not 'app' in frm.f_globals:
            raise RuntimeError('no application found')
        app = frm.f_globals['app']
    run = lambda: httpserver.serve(app, host=hostname, port=str(port))
    if reload:
        from colubrid import reloader
        reloader.main(run)
    else:
        run()
